function   fct_multipcorr(B,n,r,target_StdDp,target_Na,corr_dp,D_val,D_sim,tolerance,Niter,Nsim)
%
% this function computes the value function, optimal policy 
% multiproduct firm's pricing problem allowing correlation between the
% price gaps, and simulates it to compute its marginal distribution of price changes
% as well as the expected number of adjustment per year
%
% the content and the output that are saved are similar to the two autoexecutable m file: multipcorr.m
% and sim_marginal_p_corr.m
%
% inputs:
%%%%%%%%%
% parameters of the firm problem:
% B,n,r,target_StdDp,target_Na,corr_dp, (target_StdDp and target_Na imply
% the value of sigma?2+bar_sigma? and psi)
% D_val,D_sim, are the size of the time period for the computation of the value function and for the simulation
% to obtain statistics respectiveley,
% tolerance is the tolerance of the value function, in units of psi and 
% Niter the number of iterations of the value function and Nsim the number
% of simulations used to compute the disgtribution of price changes.
% 
% output: two mat files are saved
%%%%%%%%%
% out_nXX_corrYY.mat and simulp_nXX_corrYY.mat where
% XX is the value of n and YY the value of the correlation

D = D_val;

sum_sigma_sqr = target_Na * target_StdDp^2 ; % sum of idiosyncratic and common shock: sigma^2 + bar_sigma^2
                    % this formula makes sure that the std dev of price
                    % changes hits the target in the case of correlation
                    % zero. This uses the expression for Na, Std(Dp) and
                    % for bar y in the case of zero correlation.  
                    
sigma     = sqrt( (1-corr_dp) * sum_sigma_sqr)  ; % standard dev idiosyncratic shock
bar_sigma = sqrt(  corr_dp    * sum_sigma_sqr)  ;    % std dev common shock

psi_0 = B* target_StdDp^2/target_Na * n^2 /(2*(n+2)) ; % psi needed to match StdDp and Na and zero  correlation

B1 = B*n; n1 = 1; 

psi_1 = (B1) * target_StdDp^2/target_Na * n1^2 /(2*(n1+2)) ; % psi/B needed to match StdDp and Na with correlation one, 
                                    % which is the same as n1=1 and B1 =B*n ;
                                    
psi  = corr_dp * psi_1 + (1-corr_dp)* psi_0;
 
corr= bar_sigma^2/(bar_sigma^2+sigma^2);    

% % criterium to stop value function iterations
% Niter = 10000;    % maximum number of iterations
toler = psi*tolerance;  % minimum distance between value functions


%%%%% set up  grid of (y,z) for value function %%%%%

Ny=100; % number of points in y grid

bary_per=n*sqrt( psi*(sigma^2+bar_sigma^2)*6/(B*n) );

high_y  = 2; % highest value on the y grid, as multiple of threshold for uncorrelated case
bary    = sqrt( psi*(sigma^2+bar_sigma^2)*2*(n+2)/B );

Y_g = [0:bary*high_y/(Ny-1):bary*high_y]; %%%% this is the grid of y values

z_step = sqrt(bary*high_y/(Ny-1)*n);  %%% size of step of z values

% The next loop creates the grid of (y,z)
% points are index by (y,z)=(yg(i),zg(i)) for i=1,...,NN
i=0;
for yi=1:Ny;
    y=Y_g(yi);
    zlow  = -sqrt(y*n)*0.9999;  %%%% values of z betweens .9999 of range of domain
    zhigh =  sqrt(y*n)*0.9999;  %%%% values of z betweens .9999 of range of domain
    Nz = max(1,round((zhigh-zlow)/z_step));
    
    % ensure Nz is even  (used later)
    if Nz/2 ==round(Nz/2) 
      Nz=Nz;  
    else    
      Nz = Nz+1;
    end  
           
        zz=(zhigh-zlow)/Nz;
    for j=0:Nz
        i=i+1;
        yg(i) = y;
        zg(i) = zlow+j*zz;
    end
end
NN=length(yg);
%%%%% end of grid of (y,z) for value function %%%%%


% set of values for 3 innovations uncorrelated with mean zero and std=1
e_a = [-1 -1 -1 -1  1  1  1  1] ;
e_b = [-1 -1  1  1 -1 -1  1  1] ;
e_c = [-1  1 -1  1 -1  1 -1  1] ;

% % upper bound on value function
% v_noadj = B*yg/r + n*(sigma^2+bar_sigma^2)*B/r^2 ;
% v_noajd = min(v_noadj(1) + psi , v_noadj);
% % value of adjusting every period
% v_adj = psi/r/D;
%
%
% % inital value function
% %vg=min(v_noadj,v_adj);
%
% %%% let's start with zero value function:
% %vg=yg*0;


%%%%%%%%%%%%%%%%%%%%%% initial condition: value function from model without correlation:

sigma_old = sigma;  % temporarily uses sigma for formula of threshold
sigma = sqrt(sigma_old^2 + bar_sigma^2);


% case of uncorrelated shocks
Rbar_guess = (psi*sigma^2*2 *(n+2)/B )^0.5 ;
beta0_guess = n*sigma^2/r * 2*psi/Rbar_guess;  %  guess value function at R=zero

x0 = [beta0_guess,Rbar_guess];

options = optimset('TolFun',[1e-12],'TolX',[1e-12],'MaxFunEvals',[1000]);

[Xsol,fval, eflag] = fsolve( @(x) fun_system(x,psi,r,B,sigma,n), x0, options );

beta0 = Xsol(1);
Rbar  = Xsol(2);
pbar  = sqrt(Rbar);

beta_vec(1) =  beta0*r/(n*sigma^2);
beta_vec(2) = ( r*beta_vec(1) - B )/( 2*sigma^2*(n+2) );
i_length = 10;
% now do beta 3,4, etc
for i=2:i_length
    beta_vec(i+1) = beta_vec(i)*r/(  sigma^2* (n+2*i)*(i+1));
end

v_0 = fun_f(yg, beta0, beta_vec);  %%% creates a vector of value function in inaction
v_0 = min(beta0 + psi, v_0);       %%% value function including control region

for i=1:length(yg);
    if yg(i) > Rbar
       v_0(i) = beta0 + psi ;
    end
end


% %%% Initial condition: case of perfectly correlated shocks
% B_old = B;
% n_old = n;
% B = B*n;
% n=1;
% 
% Rbar_guess = (psi*sigma^2*2 *(n+2)/B )^0.5 ;
% beta0_guess = n*sigma^2/r * 2*psi/Rbar_guess;  %  guess value function at R=zero
% 
% x0 = [beta0_guess,Rbar_guess];
% 
% options = optimset('TolFun',[1e-12],'TolX',[1e-12],'MaxFunEvals',[1000]);
% 
% [Xsol,fval, eflag] = fsolve( @(x) fun_system(x,psi,r,B,sigma,n), x0, options );
% 
% beta0 = Xsol(1);
% Rbar  = Xsol(2);
% pbar  = sqrt(Rbar);
% 
% beta_vec(1) =  beta0*r/(n*sigma^2);
% beta_vec(2) = ( r*beta_vec(1) - B )/( 2*sigma^2*(n+2) );
% i_length = 10;
% % now do beta 3,4, etc
% for i=2:i_length
%     beta_vec(i+1) = beta_vec(i)*r/(  sigma^2* (n+2*i)*(i+1));
% end
% 
% v_0 = fun_f(yg, beta0, beta_vec);  %%% creates a vector of value function in inaction
% v_0 = min(beta0 + psi, v_0);       %%% value function including control region
% 
% % set parameters again:
% B = B_old;
% n = n_old;


sigma=sigma_old; %% set sigma to correct value

%%%%%%%%%%%%%%%%%%%%%%%%% end of intial value function

vg = v_0;  %%% initial condition is set to  vg


%%%%%%%%%%%%%%%%%%%%%%%%% value function iterations  %%%%%%%%%%%%%%%%%

k=1;
differ=toler+1;

eval=1;
nflat=1 ; %%% flatens value function every nflat evaluations at points in which v is decreasing in y
%%% if set to 1 it does it every time
%%% CONVERGENCE IS SENSITIVE TO THE CHOICE OF THIS PARAMETER
%%% for some configurations nflat=1 works well, for 

while (k < Niter) && (differ > toler ) 
    
    if k<10;  %%% fits all the points
        
        XX=[ones(1,length(zg)); yg ; yg.^2; yg.^3; zg.^2; zg.^4; yg.*(zg.^2); (yg.^2).*(zg.^2); (yg.^3).*(zg.^2)];
        
        fitv = ols(vg',XX') ;
        
    else  %%% fits polynimial only the points where value function is < max, i.e in inaction region
        
        maxv=max(vg);
        yybar= fun_ybar(zg,fitv.beta,n);
        index_inaction = find(vg < maxv + psi/100000);
        
        XX=[ones(1,length(zg)); yg; yg.^2; yg.^3; zg.^2; zg.^4; yg.*(zg.^2);(yg.^2).*(zg.^2); (yg.^3).*(zg.^2)];
        fitv = ols(vg(index_inaction)',XX(:,index_inaction)') ;
        
    end
    
    % loops value function assuming inaction at grid points
    maxv=max(vg);
    for i=1:NN
        y=yg(i);
        z=zg(i) ;
        [ yp zp ] = fun_law_motion_state(y,z,e_a,e_b,e_c,D,sigma,bar_sigma,n) ;
        Ev= mean( fun_eval_v(yp,zp,fitv.beta,psi,maxv) ) ;
        v_inaction(i) = D*B*y + exp(-D*r)*Ev;
    end
    
    vgold=vg; %%% keeps track of old value function
    vg = min( v_inaction(1) + psi , v_inaction ) ;
    k=k+1;
    
    if round(k/nflat) == eval
        %%% next lines make sure that v is not decreasing in y
        maxv=max(vg);
        for i=1:length(yg);
            if fun_eval_dvdy(yg(i),zg(i), fitv.beta) < -0.000001
                vg(i) = maxv; %%% if derivative is negative, assume it is control region
            end
        end
        eval=eval+1;
    end
    
    differ= max(abs(vg-vgold));
    
 %  k, differ
    
end
%%%%%%%%%%%%%%%%%%%%%%%%% end of value function iterations  %%%%%%%%%%%%%%%%%



%%%%%% creates a new grid for graphs
NNy=100;
bary= sqrt(psi/B*(sigma^2+bar_sigma^2)*2*(n+2));
YY_g = [0:bary*2/(NNy-1):bary*2];
zz_step = sqrt(bary*2/(NNy-1)*n);

for yi=1:NNy;
    y=YY_g(yi);
    zlow  = -sqrt(y*n);
    zhigh =  sqrt(y*n);
    NNz = max(1,round((zhigh-zlow)/zz_step));
    zzz=(zhigh-zlow)/NNz;
    for j=0:NNz
        i=i+1;
        ygg(i) = y;
        zgg(i) = zlow+j*zzz;
    end
end
NNN=length(ygg);

% value function on new grid
vgraph= fun_eval_v(ygg,zgg,fitv.beta,psi,maxv);

%%%%%% end of  new grid for graphs  %%%%%%

%%%% computes the values of (y,z) where there is control (action)
maxv=max(vgraph);
j=1;
for i=1:NNN
    if vgraph(i) >= maxv - psi/1000000;
        control(j)   = maxv;
        ygcontrol(j) = ygg(i);
        zgcontrol(j) = zgg(i);
        j=j+1;
    end
end

%  find minimum value of ybar in the contour  of control region
indx_zzero =find(zgcontrol==0);
ybar_zzero = min(ygcontrol(indx_zzero) );



fname = ['out_n', num2str(n,2),'_corr_', num2str(corr,3) ,'.mat'];

output_n.n       = n;
output_n.B       = B;
output_n.psi     = psi;
output_n.r       = r;
output_n.sigma   = sigma;
output_n.sigmabar= bar_sigma; % 
output_n.D       = D;       % length of the model time period, in years
output_n.toler   = toler ;  % tolerance for convergence of value function
output_n.nflat   = nflat ;  % index that indicates if value function is flatten every nflat periods
output_n.iter    = k     ;  % number of iteration it took to converge
output_n.beta    =  fitv.beta; % coefficients of the smooth part polinomial representation of value function
output_n.vg      =  vg;  % value function in the grid points
output_n.yg      =  yg;  % values of y in the grid
output_n.zg      =  zg;  % values of z in the grid
output_n.ybar_zzero =  ybar_zzero; % value of ybar for z=0 from the optimal policy (in grid)

save(fname, '-struct', 'output_n');


clear v0 vg zg yg zgg ygg control zgcontrol ygcontrol e_a e_b e_c

% Nsim = 25000; % number of simulations


D = D_sim;


for sim=1: Nsim
   
   % start at origin
   p = zeros(n,1);
   time = 0;
   
   y = sum(p.^2);
   z = sum(p);
   
   beta=fitv.beta; 
   
   while y < fun_ybar_adjusted(z,beta,n,ybar_zzero); 
       
       common = randn(1);
       p = p + sqrt(D)*sigma*randn(n,1) + sqrt(D)*bar_sigma*common;
       
       y = sum(p.^2);
       z = sum(p);
       
       time = time + D;
   
   end
   
   psim(1:n,sim) = p ;
   tsim(sim) = time;
   
end

if n==1    
    pvec =  psim;
    zvec =  ([psim  -psim]); % symmetrized distribution of sum of price changes
else
    pvec = vec(psim);
    zvec = sum([psim  -psim]); % symmetrized distribution of sum of price changes
end

Std_Dp = std([pvec ; -pvec]);

phist = [pvec/Std_Dp ; -pvec/Std_Dp]; % standarized (marginal) distribution of price changes

corr = bar_sigma^2/(sigma^2+bar_sigma^2);

% compute correlation between price changes
corr_Dp = cov(psim')/(mean(diag(cov(psim'))));
avg_corr_Dp = mean(corr_Dp(1,2:end));

disp([' correlation of shocks is ', num2str(corr), ' correlation of Dp is ',num2str(avg_corr_Dp) ]);

avg_time=mean(tsim);
avg_Na=1/avg_time; % average number of adjustment per year.

[density,xmesh]= ksdensity(phist,'width',0.07);


fname = ['simulp_n', num2str(n,2),'_corr_', num2str(corr,3) ,'.mat'];

sim_output.n       = n;
sim_output.sigma   = sigma;
sim_output.sigmabar= bar_sigma;
sim_output.D       = D;
sim_output.Nsim    = Nsim ;
sim_output.pvec    = pvec ;
sim_output.phist   = phist;
sim_output.avgcorrDp  = avg_corr_Dp;
sim_output.avg_Na  = avg_Na;
sim_output.StdDp   = Std_Dp;
sim_output.zvec    =  zvec ;
sim_output.xmesh   =  xmesh;
sim_output.density =  density ;
 
save(fname, '-struct', 'sim_output');
