function x = zerobess(nu,m,k)
%ZEROBESS Zeros of Bessel functions of 1st and 2nd kind.
%   X = ZEROBESS(NU,M,K) computes the M first positive zeros of the
%   Bessel function B(NU,X) of kind K, i.e. B = BESSELJ for K = 1 and 
%   B = BESSELY for K = 2. The default kind is K = 1. NU is a real
%   number in the range 0 to 1e7 and M is a nonnegative integer. 
%
%   Examples
%       x = zerobess(0,10)
%       x = zerobess(1,1e4,1);
%       x = zerobess(pi,10,2)
%       x = zerobess(1e6,5,2)
%
%   See also BESSELJ, BESSELY

%   Author: jonas.lundgren@saabgroup.com, 2010.

if nargin < 1, nu = 0; end
if nargin < 2, m = 5; end
if nargin < 3, k = 1; end

% Check nu
if numel(nu) ~= 1 || ~isfinite(nu) || ~isreal(nu) || nu < 0 || nu > 1e7
    message = 'First argument must be a real number between 0 and 1e7.';
    error('ZEROBESS:BadInput1',message)
end

% Check m
if numel(m) ~= 1 || ~isfinite(m) || ~isreal(m) || m < 0 || mod(m,1)
    message = 'Second argument must be a nonnegative integer.';
    error('ZEROBESS:BadInput2',message)
end

% Check k
switch k
    case 1
        fun = @besselj;
        c0 = [ 0.1701 -0.6563  1.0355  1.8558
               0.1608 -1.0189  3.1348  3.2447
              -0.2005 -1.2542  5.7249  4.3817 ];
    case 2
        fun = @bessely;
        c0 = [-0.0474 -0.2300  0.2409  0.9316
               0.2329 -0.8871  2.0165  2.5963
               0.0013 -1.1220  4.3729  3.8342 ];
    otherwise
        message = 'Third argument must be 1 or 2.';
        error('ZEROBESS:BadInput3',message)
end

% Initiate zeros
x = zeros(m,1);

% Initial guess for first three zeros
x0 = nu + c0*(nu+1).^[-1 -2/3 -1/3 1/3]';

% Newton-Raphson iterations
x(1:3,1) = newton(fun,nu,x0);

n = 3;	% Number of zeros computed
j = 2;	% Number of zeros to compute next
errtol = 0.005; % Relative error tolerance for initial guess

% Remaining zeros
while n < m
    
    % Upper bound for j
    j = min(j,m-n);
    
    % Predict the spacing dx between zeros
	r = diff(x(n-2:n)) - pi;
	if r(1)*r(2) > 0 && r(1)/r(2) > 1
        p = log(r(1)/r(2))/log(1-1/(n-1));
        dx = pi + r(2)*exp(p*log(1+(1:j)/(n-1)))';
    else
        dx = repmat(pi,j,1);
	end

    % Initial guess for zeros
    x0 = x(n) + cumsum(dx);

    % Newton-Raphson iterations
    x(n+1:n+j) = newton(fun,nu,x0);
    n = n + j;
 
    % Check zeros
    if ~chkzeros(nu,x(n-j-1:n))
        message = 'Bad zeros encountered. NU and/or M may be too large.';
        error('ZEROBESS:BadZeros',message)
    end
    
    % Relative error
    err = (x(n-j+1:n) - x0)./diff(x(n-j:n));

    % Number of zeros to compute next
    if max(abs(err)) < errtol
        j = 2*j;
    else
        j = 2*find(abs(err) >= errtol,1);
    end
    
end

% Return
x = x(1:m);


%--------------------------------------------------------------------------
function x = newton(fun,nu,x0)
% Newton-Raphson for Bessel function FUN with initial guess X0
x = x0;
c = 8;
for t = 1:10
    % Newton-Raphson step
    f = fun(nu,x);
    df = fun(nu-1,x) - nu*f./x;
    h = -f./df;
    x = x + h;
    % Convergence criteria
    if all(abs(h) < c*eps(x))
        break
    end
    % Relax convergence criteria
    if t >= 7
        c = 2*c;
    end
end
% Check convergence
if t == 10
    warning('ZEROBESS:Newton','No convergence for Newton-Raphson.')
end

%--------------------------------------------------------------------------
function s = chkzeros(nu,x)
% Check sequence of Bessel function zeros
dx = diff(x);
% The spacing dx should increase for nu < 0.5 and decrease otherwise
ddx = diff(dx);
if nu < 0.5
    ddx = -ddx;
end
% Criteria for acceptable zeros
s = isreal(x) && x(1) > 0 && all(dx > 3) && all(ddx < 16*eps(x(2:end-1)));
